<?php
App::uses('EvAddressBookAppModel', 'EvAddressBook.Model');
/**
 * Address Model
 *
 * @property Country $Country
 */
class Address extends EvAddressBookAppModel {

/**
 * Display field
 *
 * @var string
 */
	public $displayField = 'name';

	const US_COUNTRY_ID = 226,
		CANADA_COUNTRY_ID = 38,
		UK_COUNTRY_ID = 225;

/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'model' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				//'message' => 'Your custom message here',
				//'allowEmpty' => false,
				//'required' => false,
				//'last' => false, // Stop validation after this rule
				//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'model_id' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				//'message' => 'Your custom message here',
				//'allowEmpty' => false,
				//'required' => false,
				//'last' => false, // Stop validation after this rule
				//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'address1' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'message' => 'Please fill out address line 1'
			),
		),
		'city' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'message' => 'Please fill out the city'
			),
		),
		'post_code' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'message' => 'Please fill out the postcode'
			),
			'checkPostcode' => array(
				'rule' => array('validatePostcode'),
				'message' => 'Please enter a valid postal code'
			),
		),
		'country_id' => array(
			'notBlank' => array(
				'rule' => array('notBlank'),
				'message' => 'Please select a country'
			),
			'checkShipping' => array(
				'rule' => array('hasShipping'),
				'message' => 'This country is not available for delivery'
			),
		),
		'state' => array(
			'checkCountryState' => array(
				'rule' => array('validateCountryState'),
				'message' => 'Please select a state'
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

/**
 * belongsTo associations
 *
 * @var array
 */
	public $belongsTo = array(
		'Country' => array(
			'className' => 'EvCountry.Country'
		)
	);

	/**
	 * get all the address for a given model / model id
	 *
	 * @param 	string 	$model  	The model the data belongs to
	 * @param 	int 	$modelId 	The id the address relates to
	 * @return 	array
	 */
	public function getAddresses($model, $modelId) {
		return $this->find(
			'all',
			array(
				'conditions' => array(
					'Address.model' => $model,
					'Address.model_id' => $modelId
				),
				'contain' => array(
					'Country'
				)
			)
		);
	}

	/**
	 * get single address
	 *
	 * @param 	int 	$id 	the address id
	 * @return 	array
	 */
	public function getAddress($id) {
		return $this->find(
			'first',
			array(
				'conditions' => array(
					'Address.id' => $id
				),
				'contain' => array(
					'Country'
				)
			)
		);
	}

/**
 * Validate US states. If the country provided isn't US then it passes. The state must be provided and be a length of 2
 * uppercase letters.
 */
	public function validateCountryState($check) {
		//Check the country selected is the US
		if ($this->data['Address']['country_id'] == self::US_COUNTRY_ID ||
			$this->data['Address']['country_id'] == self::CANADA_COUNTRY_ID
		) {
			//Check that a state has been supplied
			if (isset($this->data['Address']['state']) && !empty($this->data['Address']['state'])) {
				//Check if it is an uppercase 2 character string for US state code
				if (!preg_match('/^[A-Z]{2}$/', $this->data['Address']['state'])) {
					return false;
				}
			} else {
				//State hasn't been supplied
				return false;
			}
		}

		return true;
	}

/**
 * Validate  postcodes. If the country provided isn't US, CA or UK then it passes.
 */
	public function validatePostcode($check) {
		// Only validating postcodes on production as test payments can require specific test address details
		if (Configure::read('app.environment') !== 'DEVELOPMENT' || Configure::read('EvAddressBook.validatePostcodeInDevelopment')) {
			if ($this->data['Address']['country_id'] == self::US_COUNTRY_ID) {
				/**
				 * US Postcode Validation
				 * - Checks for a 5 digit number
				 */
				if (!preg_match('/^\d{5}$/', $this->data['Address']['post_code'])) {
					return false;
				}
			} elseif ($this->data['Address']['country_id'] == self::CANADA_COUNTRY_ID) {
				/**
				 * Canada Postcode Validation
				 * - Checks for a Regex match.
				 */
				if (! preg_match('/^(\d{5}-\d{4}|\d{5}|\d{9})$|^([a-zA-Z]\d[a-zA-Z]( )?\d[a-zA-Z]\d)$/', $this->data['Address']['post_code'])) {
					return false;
				}

			} elseif ($this->data['Address']['country_id'] == self::UK_COUNTRY_ID) {
				/**
				 * UK Postcode Validation
				 * - Checks for a Regex match. NOTE: Does not support oversees teritories
				 * - Supports postcodes with and without spaces
				 */
				if (! preg_match('/^(([gG][iI][rR] {0,}0[aA]{2})|((([a-pr-uwyzA-PR-UWYZ][a-hk-yA-HK-Y]?[0-9][0-9]?)|(([a-pr-uwyzA-PR-UWYZ][0-9][a-hjkstuwA-HJKSTUW])|([a-pr-uwyzA-PR-UWYZ][a-hk-yA-HK-Y][0-9][abehmnprv-yABEHMNPRV-Y]))) {0,}[0-9][abd-hjlnp-uw-zABD-HJLNP-UW-Z]{2}))$/', $this->data['Address']['post_code'])) {
					return false;
				}
			}
		}

		return true;
	}

/**
 * Validate the country to see if any shipping is available for the current address or not.
 * Can be skipped if config setting, `checkCountryForShipping`, is set to false
 */
	public function hasShipping($check) {
		if (CakePlugin::loaded('EvShipping') && Configure::read('EvAddressBook.checkCountryForShipping')) {
			$ShippingRule = EvClassRegistry::init('EvShipping.ShippingRule');
			$rule = $ShippingRule->find(
				'first',
				[
					'joins' => [
						[
							'table' => 'ev_shipping_shipping_rules_zones',
							'alias' => 'ShippingRuleZone',
							'conditions' => [
								'ShippingRuleZone.shipping_rule_id = ShippingRule.id'
							]
						],
						[
							'table' => 'ev_shipping_zones',
							'alias' => 'Zone',
							'conditions' => [
								'Zone.id = ShippingRuleZone.zone_id',
								'Zone.is_active' => true
							]
						],
						[
							'table' => 'ev_shipping_zones_countries',
							'alias' => 'ZoneCountry',
							'conditions' => [
								'ZoneCountry.zone_id = Zone.id',
								'ZoneCountry.country_id' => $this->data['Address']['country_id']
							]
						]
					],
					'conditions' => [
						'ShippingRule.is_active' => true
					]
				]
			);

			return !empty($rule);
		}

		return true;
	}
}
